-- PeonQuestComplete addon
--  Provides an audio cue on the following events:
--  - completing a quest
--  - completing one of the objectives of a quest (without fully completing that quest)
-- Based on Yoco's QuestSounds http://www.curse.com/addons/wow/questsounds-yoco
-- $Date: 2013/10/30 18:10 $ $Revision: 6.0.3.19103 $

-- Sound set definitions
local soundSets = {
-- Plays sounds on quest completeion and objective completion.
  mix = {
	questComplete = "Sound/Creature/Peon/PeonBuildingComplete1.ogg",
	objectiveComplete = "Sound/Creature/Peasant/PeasantReady1.ogg",
	objectiveProgress = "",
  },
  peasant = {
	questComplete = "Sound/Creature/Peasant/PeasantReady1.ogg",
	objectiveComplete = "Sound/Creature/Peasant/PeasantYes1.ogg",
	objectiveProgress = "",
  },  
  peon = { -- this set acts as default for new installations
	questComplete = "Sound/Creature/Peon/PeonBuildingComplete1.ogg",
	objectiveComplete = "Sound/Creature/Peon/PeonReady1.ogg",
	objectiveProgress = "",
  },
-- Plays sounds on quest completeion only.
  peasantquestonly = {
	questComplete = "Sound/Creature/Peasant/PeasantReady1.ogg",
	objectiveComplete = "",
	objectiveProgress = "",
  },
  peonquestonly = {
	questComplete = "Sound/Creature/Peon/PeonBuildingComplete1.ogg",
	objectiveComplete = "",
	objectiveProgress = "",
  }, 
-- Plays sounds on quest completeion, objective completion and objectitive progress (e.g. 1 of 10)
  mixall = {
	questComplete = "Sound/Creature/Peon/PeonBuildingComplete1.ogg",
	objectiveComplete = "Sound/Creature/Peasant/PeasantReady1.ogg",
	objectiveProgress = "Sound/Creature/Peon/PeonYes3.ogg",
  },
  peasantall = {
	questComplete = "Sound/Creature/Peasant/PeasantReady1.ogg",
	objectiveComplete = "Sound/Creature/Peasant/PeasantYes4.ogg",
	objectiveProgress = "Sound/Creature/Peasant/PeasantYes1.ogg",
  },  
  peonall = {
	questComplete = "Sound/Creature/Peon/PeonBuildingComplete1.ogg",
	objectiveComplete = "Sound/Creature/Peon/PeonReady1.ogg",
	objectiveProgress = "Sound/Creature/Peon/PeonYes3.ogg",
  },
}

-- The saved variables
PQCSndVars = {} -- will be initialized in PLAYER_LOGIN

-- The sounds to play
local sounds = nil -- will be initialized in PLAYER_LOGIN

-- The message level codes
local messageLevels = {
  -- increment in multiples of 10 so we can add levels later without breaking stuff
  none = 0,
  completion = 10,
  objectives = 20,
  all = 30
}

local me = CreateFrame("Frame")
local events = {}

me.questIndex = 0
me.questId = 0
me.completeCount = 0
me.msgLvl = messageLevels.all

local function countCompleteObjectives(index)
  local n = 0
  for i = 1, GetNumQuestLeaderBoards(index) do
    local _, _, finished = GetQuestLogLeaderBoard(i, index)
    if finished then
      n = n + 1
    end
  end
  return n
end

function me:setQuest(index)
  self.questIndex = index
  if index>0 then
    local _, _, _, _, _, _, _, id = GetQuestLogTitle(index)
    self.questId = id
    if id and id>0 then
      self.completeCount = countCompleteObjectives(index)
    end
  end
end

function me:checkQuest()
  if self.questIndex>0 then
    local index = self.questIndex
    self.questIndex = 0
    -- Beware! One of the output arguments was removed in WoD
    local title, level, _, _, _, complete, daily, id = GetQuestLogTitle(index)
    local link = GetQuestLink(id)
    if id==self.questId then
      if id and id>0 then
        local objectivesComplete = countCompleteObjectives(index)
        if complete then
          if me.msgLvl >= messageLevels.completion then
            print("["..level.."] '"..link.."': complete");
          end
          self:Play(sounds and sounds.questComplete)
        elseif objectivesComplete>self.completeCount then
          if me.msgLvl >= messageLevels.objectives then
            print("["..level.."] '"..link.."': objective complete ("..objectivesComplete.."/"..GetNumQuestLeaderBoards(index)..")");
          end
          self:Play(sounds and sounds.objectiveComplete)
        else
          if me.msgLvl >= messageLevels.all then
--             print("["..level.."] '"..link.."': updated");
          end
          self:Play(sounds and sounds.objectiveProgress)
        end
      end
    end
  end
end

function me:init()
  self:SetScript("OnEvent", function(frame, event, ...)
    local handler = events[event]
    if handler then
      -- dispatch events that were auto-registered by naming convention
      handler(frame, ...)
    end
  end)
  for k,v in pairs(events) do
    self:RegisterEvent(k)
  end
end

function me:Play(sound)
  if sound and sound~="" then
    -- print("PeonQuestComplete debug message: Playing:", sound)
    PlaySoundFile(sound);
  end
end

function me:SetSoundSet(setName, silent)
  if setName and soundSets[setName] then
    PQCSndVars.ActiveSet = setName
    sounds = soundSets[setName]
    if not(silent) then
      print("Switching to sound set |cFF44FF00"..setName.."|r")
      me:Play(sounds.questComplete)
    end
  else
    print("Recognized sound set names are:")
    for k,v in pairs(soundSets) do
      if k==PQCSndVars.ActiveSet then
        print("  "..k.."      (*)")
      else
        print("  "..k)
      end
    end
  end
end

function me:SetMessageLevel(messageLevel)
  local levelCode = messageLevels[messageLevel]
  if levelCode==nil then
    -- ignore silently now
  else
    PQCSndVars.Messages = messageLevel
    me.msgLvl = levelCode
  end
end


function me:SetMessageLevelCmd(messageLevel)
  if messageLevel==nil or messageLevel=="" then
    print("The current message verbosity level is '"..PQCSndVars.Messages.."'")
  else
    local levelCode = messageLevels[messageLevel]
    if levelCode==nil then
      print("Unrecognized message verbosity level '"..messageLevel.."'. Expecting one of :")
      for k,v in pairs(messageLevels) do
        print("  "..k);
      end
    else
      me:SetMessageLevel(messageLevel)
      print("Setting message verbosity to "..PQCSndVars.Messages)
    end
  end
end


-- ................................................................
-- event handlers

function events:UNIT_QUEST_LOG_CHANGED(unit)
  if unit=="player" then
    me:checkQuest()
  end
end

function events:QUEST_WATCH_UPDATE(index)
  -- This event triggers just *before* the changes are registered
  -- in the quest log, giving a great opportunity to quantify changes
  -- in the subsequent UNIT_QUEST_LOG_CHANGED
  me:setQuest(index)
end

function events:PLAYER_LOGIN()
  -- Saved variables are stable now
  if not(PQCSndVars.ActiveSet) then
    PQCSndVars.ActiveSet = "peon"
  end
  if not(PQCSndVars.Messages) then
    PQCSndVars.Messages = "objectives"
    -- one of "objectives", "completion", "none"
  end
  me:SetMessageLevel(PQCSndVars.Messages)
  me:SetSoundSet(PQCSndVars.ActiveSet, true)
end

-- ................................................................
-- command interface

SLASH_PEONQUESTCOMPLETE1 = "/pqc"
SLASH_PEONQUESTCOMPLETE2 = "/peonquestcomplete"

function SlashCmdList.PEONQUESTCOMPLETE(msg, editbox)
  local cmd, rest = msg:match("^(%S*)%s*(.-)$")
  me:Command(cmd, rest)
end

function me:Command(command, arg)
  local handler = self["CMD"..command]
  if handler then
    handler(self, arg)
  else
    if command~="" then
      print("Unknown peonquestcomplete command '"..command.."'");
    end
    print("  /pqc set <soundsetname>")
    print("     Sets the sound set and plays its quest completion sound")
    print("  /pqc set")
    print("     Lists the known sound set names")
    print("  /pqc test <quest|objective|part>")
    print("     Plays the current 'quest', 'objective' or 'part' completion sound")
    print("  /pqc message <objectives|completion|none>")
    print("     Sets verbosity of messages in the chat log to 'objectives', 'completion' or 'none'")
    print("  The current sound set is |cFF44FF00"..PQCSndVars.ActiveSet.."|r");
    print("  The current message verbosity is |cFF44FF00"..PQCSndVars.Messages.."|r");
  end
end

function me:CMDtest(arg)
  if arg=="1" or arg=="quest" then
    print("Playing |cFFCCCC44quest complete|r sound of sound set |cFF44FF00"..PQCSndVars.ActiveSet.."|r")
    self:Play(sounds and sounds.questComplete)
  elseif arg=="2" or arg=="objective" then
    print("Playing |cFFCCCC44objective complete|r sound of sound set |cFF44FF00"..PQCSndVars.ActiveSet.."|r")
    self:Play(sounds and sounds.objectiveComplete)
  elseif arg=="3" or arg=="part" then
    print("Playing |cFFCCCC44objective progress|r sound of sound set |cFF44FF00"..PQCSndVars.ActiveSet.."|r")
    self:Play(sounds and sounds.objectiveProgress)
  else
    print("  Usage: /pqc test <sound id>")
    print("    Recognized values for <sound id> are:")
    print("      '1' or 'quest' for the quest complete sound")
    print("      '2' or 'objective' for the objective complete sound")
    print("      '3' or 'part' for the objective part update sound")
  end
end

function me:CMDset(arg)
  self:SetSoundSet(arg)
end

function me:CMDmessage(arg)
  self:SetMessageLevelCmd(arg)
end

function me:CMDmsg(arg)
  self:SetMessageLevelCmd(arg)
end

-- ................................................................
-- must be last line:
me:init()
